/*----------------------------------------------------------------------*/
//  am2302.c
//
//  Created on: 22.02.2013
//	Author: Thomas Wedemeyer
//
//	Simple functions to read the AM2303 / DHT22 temperatur/humidity sensor with
//	the SystemTick-Timer of the LPC1769
/*----------------------------------------------------------------------*/

#include "define.h"
#include "LPC17xx.h"
#include "am2302.h"
#include "types.h"

/*----------------------------------------------------------------------*/
//! Setup of the IO-pin for the AM2302-interface
/*----------------------------------------------------------------------*/
void am2303Init()
{
	LPC_PINCON->PINMODE4 &= ~(PIN_MASK_HTDATA);		// Pullup enabled
	AM2302_PORT->FIOSET = PIN_MASK_HTDATA;			// Set default state = High
	LPC_PINCON->PINMODE_OD2 |= PIN_MASK_HTDATA;		// Open Drain Mode for IO
	AM2302_PORT->FIODIR	|= PIN_MASK_HTDATA;			// Defaultstate output

}


/*----------------------------------------------------------------------*/
//! Readfunction for the AM2302
//! This function use the SystemTick-Timer for the decoding of the
//! sensor timing. The timing is calculated in espect of the SystemClk.
//! @param	buffer	Pointer to the data structur of sensor data
//! @return	TRUE = new data, FALSE = no data / error
/*----------------------------------------------------------------------*/
int32_t	am2302Read(struct am2303_data * buffer)
{
	uint8_t			data[5];
	uint32_t		system_time;
	uint32_t		i=0;
	uint32_t		time_zero, time_start,pulse_length;
	uint32_t		tick = 0;
	uint32_t		state = AM2302_HIGHSTATE;
	uint8_t			checksum;
	uint32_t		timeout = 0;

	for(i=0;i<5;i++)
	{
		data[i]=0;
	}

	// Calculate the timing in respect of the SystemCoreClock
	time_start = (SystemCoreClock / AM2302_STARTTRIGGER)+100;
	time_zero = SystemCoreClock /AM2302_ZERO;  //10000;

	AM2302_PORT->FIODIR	|= PIN_MASK_HTDATA;			// Defaultstate output
	AM2302_PORT->FIOCLR = PIN_MASK_HTDATA;			// Set low
	// Wait 500µs
	tick = SysTick->VAL;
	do{

		system_time =  SysTick->VAL;
		if(system_time > tick)
		{
			if((SysTick->LOAD - system_time + tick)> time_start)
				break;
		}
		else
		{
			if((tick - system_time) > time_start)
				break;
		}
	} while(TRUE);
	AM2302_PORT->FIOSET = PIN_MASK_HTDATA;			// Set High
	AM2302_PORT->FIODIR	&= ~PIN_MASK_HTDATA;		// Defaultstate input

	tick = SysTick->VAL;
	i = 0;
	do
	{
		timeout++;	// Simple timeout
		if(timeout > 0x2000)
			return FALSE;

		switch(state)
		{
			case AM2302_LOWSTATE:	// Wait for high level
				if((AM2302_PORT->FIOPIN & PIN_MASK_HTDATA) != 0)
				{
					state = AM2302_HIGHSTATE;
					timeout = 0;
				}
				break;

			case AM2302_HIGHSTATE: // Wait for LOWlevel
				if((AM2302_PORT->FIOPIN & PIN_MASK_HTDATA) == 0)
				{
					state = AM2302_LOWSTATE;
					system_time =  SysTick->VAL;
					if(system_time < tick)
					{
						pulse_length = (tick - system_time);
					}
					else
					{
						pulse_length = (SysTick->LOAD - system_time + tick);
					}

					tick = system_time;
					if(i>1)			// don't save the prebit
					{
						data[(i-2)/8] = data[(i-2)/8] << 1;
						if(pulse_length > time_zero)
							data[(i-2)/8] |= 0x01;
					}
					i++;
					timeout = 0;
				}
				break;
		}
	} while (i < 42);

	//Check data
	checksum = data[0] + data[1] + data[2] + data[3];
	if(checksum != data[4])
		return FALSE;

	// Calculate Temperatur  and copy to buffer
	buffer->temperatur = data[2] & 0x7F;
	buffer->temperatur *= 256;
	buffer->temperatur += data[3];
	if(data[2] & 0x80)
		buffer->temperatur *= -1;

	// calculate humidity and copy to buffer
	buffer->humidity = data[0];
	buffer->humidity *= 256;
	buffer->humidity += data[1];

	return TRUE;
}

